@@ -8,8 +8,8 @@
  * \note	This program is free software and may be modified and distributed under the terms of the GNU Public License.
  *		See the LICENSE file at the top of the source tree.
  *
- * $Date: 2014-05-30 19:56:13 +0200 (Fri, 30 May 2014) $
- * $Revision: 5449 $  
+ * $Date: 2014-11-05 16:21:05 +0100 (Wed, 05 Nov 2014) $
+ * $Revision: 5789 $  
  */
 #ifndef __PBX_IMPL_H
 #define __PBX_IMPL_H
@@ -60,7 +60,7 @@ struct sccp_pbx_cb {
 	const char *(*const getChannelMacroContext) (const sccp_channel_t * channel);
 	void (*const setChannelMacroContext) (const sccp_channel_t *channel, const char *linkedid);
 	const char *(*const getChannelCallForward) (const sccp_channel_t * channel);
-	void (*const setChannelCallForward) (const sccp_channel_t *channel, const char *linkedid);
+	void (*const setChannelCallForward) (const sccp_channel_t *channel, const char *fwdnum);
 
 	/** get channel by name */
 	boolean_t(*const getChannelByName) (const char *name, PBX_CHANNEL_TYPE **pbx_channel);
@@ -76,6 +76,9 @@ struct sccp_pbx_cb {
 
 	int (*const sched_add) (int when, sccp_sched_cb callback, const void *data);
 	int (*const sched_del) (int id);
+	int (*const sched_add_ref) (int *id, int when, sccp_sched_cb callback, sccp_channel_t *channel);
+	int (*const sched_del_ref) (int *id, const sccp_channel_t *channel);
+	int (*const sched_replace_ref) (int *id, int when, ast_sched_cb callback, sccp_channel_t *channel);
 	long (*const sched_when) (int id);
 	int (*const sched_wait) (int id);
 
@@ -161,3 +164,4 @@ struct sccp_pbx_cb {
 };
 
 #endif
+// kate: indent-width 8; replace-tabs off; indent-mode cstyle; auto-insert-doxygen on; line-numbers on; tab-indents on; keep-extra-spaces off; auto-brackets off;
