@@ -1,15 +1,15 @@
 
 /*!
- * \file	sccp_indicate.c
- * \brief	SCCP Indicate Class
- * \author	Sergio Chersovani <mlists [at] c-net.it>
- * \note	Reworked, but based on chan_sccp code.
- * 		The original chan_sccp driver that was made by Zozo which itself was derived from the chan_skinny driver.
- *		Modified by Jan Czmok and Julien Goodwin
- * \note	This program is free software and may be modified and distributed under the terms of the GNU Public License.
+ * \file        sccp_indicate.c
+ * \brief       SCCP Indicate Class
+ * \author      Sergio Chersovani <mlists [at] c-net.it>
+ * \note        Reworked, but based on chan_sccp code.
+ *              The original chan_sccp driver that was made by Zozo which itself was derived from the chan_skinny driver.
+ *              Modified by Jan Czmok and Julien Goodwin
+ * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
  *
- * $Date: 2014-08-07 12:11:28 +0200 (Thu, 07 Aug 2014) $
- * $Revision: 5628 $
+ * $Date: 2014-11-05 16:40:01 +0100 (Wed, 05 Nov 2014) $
+ * $Revision: 5790 $
  */
 
 #include <config.h>
@@ -22,9 +22,9 @@
 #include "sccp_conference.h"
 #include "sccp_actions.h"
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5628 $")
+SCCP_FILE_VERSION(__FILE__, "$Revision: 5790 $")
 
-static void __sccp_indicate_remote_device(sccp_device_t * device, sccp_channel_t * c, sccp_line_t * line, uint8_t state);
+static void __sccp_indicate_remote_device(sccp_device_t * device, sccp_channel_t * c, sccp_line_t * line, sccp_channelstate_t state);
 
 /*!
  * \brief Indicate Without Lock
@@ -54,18 +54,21 @@ void __sccp_indicate(sccp_device_t * device, sccp_channel_t * c, sccp_channelsta
 	}
 
 	AUTO_RELEASE sccp_device_t *d = (device) ? sccp_device_retain(device) : sccp_channel_getDevice_retained(c);
+
 	if (!d) {
 		sccp_log((DEBUGCAT_INDICATE)) (VERBOSE_PREFIX_1 "SCCP: The channel %d does not have a device\n", c->callid);
 		return;
 	}
 
 	AUTO_RELEASE sccp_line_t *l = sccp_line_retain(c->line);
+
 	if (!l) {
 		pbx_log(LOG_ERROR, "SCCP: The channel %d does not have a line\n", c->callid);
 		return;
 	}
 
 	AUTO_RELEASE sccp_linedevices_t *linedevice = sccp_linedevice_find(d, l);
+
 	if (linedevice) {
 		instance = linedevice->lineInstance;
 	} else {
@@ -75,7 +78,7 @@ void __sccp_indicate(sccp_device_t * device, sccp_channel_t * c, sccp_channelsta
 	/* all the check are ok. We can safely run all the dev functions with no more checks */
 	sccp_log((DEBUGCAT_INDICATE + DEBUGCAT_DEVICE + DEBUGCAT_LINE)) (VERBOSE_PREFIX_3 "%s: Indicate SCCP state %d (%s),channel state %d (%s) on call %s-%08x (previous channelstate %d (%s))\n", d->id, state, sccp_channelstate2str(state), c->state, sccp_channelstate2str(c->state), l->name, c->callid, c->previousChannelState, sccp_channelstate2str(c->previousChannelState));
 	sccp_channel_setChannelstate(c, state);
-	
+
 	switch (state) {
 		case SCCP_CHANNELSTATE_DOWN:
 			//PBX(set_callstate)(c, AST_STATE_DOWN);
@@ -89,8 +92,9 @@ void __sccp_indicate(sccp_device_t * device, sccp_channel_t * c, sccp_channelsta
 				sccp_dev_set_cplane(d, instance, 1);
 				sccp_dev_displayprompt(d, instance, c->callid, SKINNY_DISP_ENTER_NUMBER, GLOB(digittimeout));
 				sccp_dev_set_keyset(d, instance, c->callid, KEYMODE_OFFHOOK);
-				if (d->earlyrtp != SCCP_EARLYRTP_IMMEDIATE)
+				if (d->earlyrtp != SCCP_EARLYRTP_IMMEDIATE) {
 					sccp_dev_starttone(d, SKINNY_TONE_INSIDEDIALTONE, instance, c->callid, 0);
+				}
 			} else {										// call pickup
 				sccp_dev_set_speaker(d, SKINNY_STATIONSPEAKER_ON);
 				sccp_device_setLamp(d, SKINNY_STIMULUS_LINE, instance, SKINNY_LAMP_ON);
@@ -129,7 +133,7 @@ void __sccp_indicate(sccp_device_t * device, sccp_channel_t * c, sccp_channelsta
 			break;
 		case SCCP_CHANNELSTATE_ONHOOK:
 			c->state = SCCP_CHANNELSTATE_DOWN;
-			//d->indicate->onhook(d, c, instance, c->callid);			// reverted to old code, new one does not seem to work
+			//d->indicate->onhook(d, c, instance, c->callid);                       // reverted to old code, new one does not seem to work
 			sccp_dev_stoptone(d, instance, c->callid);
 			sccp_device_setLamp(d, SKINNY_STIMULUS_LINE, instance, SKINNY_LAMP_OFF);
 			sccp_dev_cleardisplaynotify(d);
@@ -155,15 +159,15 @@ void __sccp_indicate(sccp_device_t * device, sccp_channel_t * c, sccp_channelsta
 			d->protocol->sendDialedNumber(d, c);
 			sccp_dev_displayprompt(d, instance, c->callid, SKINNY_DISP_RING_OUT, GLOB(digittimeout));
 			d->protocol->sendCallInfo(d, c, instance);
-			if (!c->rtp.audio.rtp) {
-				if (d->earlyrtp <= SCCP_EARLYRTP_RINGOUT) {
-					sccp_channel_openReceiveChannel(c);
-				} else {
-					sccp_dev_starttone(d, (uint8_t) SKINNY_TONE_ALERTINGTONE, instance, c->callid, 0);
-				}
+
+			if (d->earlyrtp <= SCCP_EARLYRTP_RINGOUT && c->rtp.audio.writeState == SCCP_RTP_STATUS_INACTIVE) {
+				sccp_channel_openReceiveChannel(c);
+			}
+			if (c->rtp.audio.writeState == SCCP_RTP_STATUS_INACTIVE) {				/* send tone if ther is no rtp for inband signaling */
+				sccp_dev_starttone(d, (uint8_t) SKINNY_TONE_ALERTINGTONE, instance, c->callid, 0);
 			}
 			sccp_dev_set_keyset(d, instance, c->callid, KEYMODE_RINGOUT);
-			
+
 			PBX(set_callstate) (c, AST_STATE_RING);
 			break;
 		case SCCP_CHANNELSTATE_RINGING:
@@ -200,11 +204,12 @@ void __sccp_indicate(sccp_device_t * device, sccp_channel_t * c, sccp_channelsta
 			}
 
 			sccp_dev_set_keyset(d, instance, c->callid, KEYMODE_RINGIN);
-
-			char prompt[100];
-
-			snprintf(prompt, sizeof(prompt), "%s" SKINNY_DISP_FROM ": %s", (c->ringermode == SKINNY_RINGTYPE_URGENT) ? SKINNY_DISP_FLASH : "", strlen(c->callInfo.callingPartyName) ? c->callInfo.callingPartyName : c->callInfo.callingPartyNumber);
-			sccp_dev_displayprompt(d, instance, c->callid, prompt, GLOB(digittimeout));
+			/*                      char prompt[100];
+			   snprintf(prompt, sizeof(prompt), "%s%s: %s", (c->ringermode == SKINNY_RINGTYPE_URGENT) ? SKINNY_DISP_FLASH : "", strlen(c->callInfo.originalCalledPartyName) ? c->callInfo.originalCalledPartyName : SKINNY_DISP_FROM, strlen(c->callInfo.callingPartyName) ? c->callInfo.callingPartyName : c->callInfo.callingPartyNumber);
+			   sccp_dev_displayprompt(d, instance, c->callid, prompt, GLOB(digittimeout)); */
+			if (c->ringermode) {
+				sccp_dev_displayprompt(d, instance, c->callid, SKINNY_DISP_FLASH, GLOB(digittimeout));
+			}
 
 			PBX(set_callstate) (c, AST_STATE_RINGING);						/*!\todo thats not the right place to update pbx state */
 			break;
@@ -220,7 +225,7 @@ void __sccp_indicate(sccp_device_t * device, sccp_channel_t * c, sccp_channelsta
 			sccp_dev_set_keyset(d, instance, c->callid, KEYMODE_CONNECTED);
 			break;
 		case SCCP_CHANNELSTATE_BUSY:
-			if (!c->rtp.audio.rtp) {
+			if (c->rtp.audio.writeState == SCCP_RTP_STATUS_INACTIVE) {
 				sccp_dev_starttone(d, SKINNY_TONE_LINEBUSYTONE, instance, c->callid, 0);
 			}
 			sccp_dev_displayprompt(d, instance, c->callid, SKINNY_DISP_BUSY, GLOB(digittimeout));
@@ -270,22 +275,23 @@ void __sccp_indicate(sccp_device_t * device, sccp_channel_t * c, sccp_channelsta
 			break;
 		case SCCP_CHANNELSTATE_CONGESTION:
 			/* it will be emulated if the rtp audio stream is open */
-			if (!c->rtp.audio.rtp) {
+			if (c->rtp.audio.writeState == SCCP_RTP_STATUS_INACTIVE) {
 				sccp_dev_starttone(d, SKINNY_TONE_REORDERTONE, instance, c->callid, 0);
 			}
 			d->protocol->sendCallInfo(d, c, instance);
 			sccp_dev_displayprompt(d, instance, c->callid, SKINNY_DISP_TEMP_FAIL, GLOB(digittimeout));
 			// wait 15 seconds, then hangup automatically
-			c->scheduler.hangup = sccp_sched_add(15000, sccp_channel_sched_endcall_by_callid, &c->callid);
+			sccp_channel_schedule_hangup(c, SCCP_HANGUP_TIMEOUT);
 			break;
 		case SCCP_CHANNELSTATE_CALLWAITING:
 			{
 				{
 					/* When dialing a shared line which you also have registered, we don't want to outgoing call to show up on our own device as a callwaiting call */
 					AUTO_RELEASE sccp_channel_t *activeChannel = sccp_device_getActiveChannel(d);
+
 					if (activeChannel && (sccp_strequals(PBX(getChannelLinkedId) (activeChannel), PBX(getChannelLinkedId) (c)))) {
-						sccp_log(DEBUGCAT_INDICATE)(VERBOSE_PREFIX_3 "%s: (SCCP_CHANNELSTATE_CALLWAITING) Already Own Part of the Call: Skipping\n", DEV_ID_LOG(d));
-						sccp_log_and(DEBUGCAT_INDICATE + DEBUGCAT_HIGH)(VERBOSE_PREFIX_3 "%s: LinkedId: %s / %s: LinkedId Remote: %s\n", DEV_ID_LOG(d), PBX(getChannelLinkedId)(c), DEV_ID_LOG(d), PBX(getChannelLinkedId)(activeChannel));
+						sccp_log(DEBUGCAT_INDICATE) (VERBOSE_PREFIX_3 "%s: (SCCP_CHANNELSTATE_CALLWAITING) Already Own Part of the Call: Skipping\n", DEV_ID_LOG(d));
+						sccp_log_and(DEBUGCAT_INDICATE + DEBUGCAT_HIGH) (VERBOSE_PREFIX_3 "%s: LinkedId: %s / %s: LinkedId Remote: %s\n", DEV_ID_LOG(d), PBX(getChannelLinkedId) (c), DEV_ID_LOG(d), PBX(getChannelLinkedId) (activeChannel));
 						break;
 					}
 				}
@@ -307,11 +313,11 @@ void __sccp_indicate(sccp_device_t * device, sccp_channel_t * c, sccp_channelsta
 		case SCCP_CHANNELSTATE_CALLTRANSFER:
 			sccp_dev_displayprompt(d, instance, c->callid, SKINNY_DISP_TRANSFER, GLOB(digittimeout));
 			sccp_dev_set_ringer(d, SKINNY_RINGTYPE_OFF, instance, c->callid);
-			sccp_device_sendcallstate(d, instance, c->callid, SCCP_CHANNELSTATE_CALLTRANSFER, SKINNY_CALLPRIORITY_LOW, SKINNY_CALLINFO_VISIBILITY_DEFAULT);
+			sccp_device_sendcallstate(d, instance, c->callid, SKINNY_CALLSTATE_CALLTRANSFER, SKINNY_CALLPRIORITY_LOW, SKINNY_CALLINFO_VISIBILITY_DEFAULT);
 			d->protocol->sendCallInfo(d, c, instance);
 			break;
 		case SCCP_CHANNELSTATE_CALLCONFERENCE:
-			sccp_device_sendcallstate(d, instance, c->callid, SCCP_CHANNELSTATE_CALLCONFERENCE, SKINNY_CALLPRIORITY_LOW, SKINNY_CALLINFO_VISIBILITY_DEFAULT);
+			//    sccp_device_sendcallstate(d, instance, c->callid, SCCP_CHANNELSTATE_CALLCONFERENCE, SKINNY_CALLPRIORITY_LOW, SKINNY_CALLINFO_VISIBILITY_DEFAULT);
 			break;
 		case SCCP_CHANNELSTATE_INVALIDCONFERENCE:
 			/*! \todo SCCP_CHANNELSTATE_INVALIDCONFERENCE To be implemented */
@@ -319,18 +325,18 @@ void __sccp_indicate(sccp_device_t * device, sccp_channel_t * c, sccp_channelsta
 			break;
 		case SCCP_CHANNELSTATE_CONNECTEDCONFERENCE:
 			sccp_log((DEBUGCAT_INDICATE)) (VERBOSE_PREFIX_3 "%s: SCCP_CHANNELSTATE_CONNECTEDCONFERENCE (%s)\n", d->id, sccp_channelstate2str(c->previousChannelState));
-/*
-			sccp_dev_set_ringer(d, SKINNY_RINGTYPE_OFF, instance, c->callid);
-			sccp_dev_set_speaker(d, SKINNY_STATIONSPEAKER_ON);
-			sccp_dev_stoptone(d, instance, c->callid);
-			sccp_device_setLamp(d, SKINNY_STIMULUS_LINE, instance, SKINNY_LAMP_ON);
-			sccp_device_sendcallstate(d, instance, c->callid, SKINNY_CALLSTATE_CONNECTED, SKINNY_CALLPRIORITY_LOW, SKINNY_CALLINFO_VISIBILITY_DEFAULT);
-			d->protocol->sendCallInfo(d, c, instance);
-			sccp_dev_set_cplane(d, instance, 1);
-			sccp_dev_set_keyset(d, instance, c->callid, KEYMODE_CONNCONF);
-			sccp_dev_displayprompt(d, instance, c->callid, SKINNY_DISP_CONNECTED, GLOB(digittimeout));
-*/
-// replaced by
+			/*
+			   sccp_dev_set_ringer(d, SKINNY_RINGTYPE_OFF, instance, c->callid);
+			   sccp_dev_set_speaker(d, SKINNY_STATIONSPEAKER_ON);
+			   sccp_dev_stoptone(d, instance, c->callid);
+			   sccp_device_setLamp(d, SKINNY_STIMULUS_LINE, instance, SKINNY_LAMP_ON);
+			   sccp_device_sendcallstate(d, instance, c->callid, SKINNY_CALLSTATE_CONNECTED, SKINNY_CALLPRIORITY_LOW, SKINNY_CALLINFO_VISIBILITY_DEFAULT);
+			   d->protocol->sendCallInfo(d, c, instance);
+			   sccp_dev_set_cplane(d, instance, 1);
+			   sccp_dev_set_keyset(d, instance, c->callid, KEYMODE_CONNCONF);
+			   sccp_dev_displayprompt(d, instance, c->callid, SKINNY_DISP_CONNECTED, GLOB(digittimeout));
+			 */
+			// replaced by
 			if (linedevice) {
 				d->indicate->connected(d, linedevice, c);
 			}
@@ -344,7 +350,7 @@ void __sccp_indicate(sccp_device_t * device, sccp_channel_t * c, sccp_channelsta
 
 			break;
 		case SCCP_CHANNELSTATE_CALLPARK:
-			sccp_device_sendcallstate(d, instance, c->callid, SCCP_CHANNELSTATE_CALLPARK, SKINNY_CALLPRIORITY_LOW, SKINNY_CALLINFO_VISIBILITY_DEFAULT);
+			sccp_device_sendcallstate(d, instance, c->callid, SKINNY_CALLSTATE_CALLPARK, SKINNY_CALLPRIORITY_LOW, SKINNY_CALLINFO_VISIBILITY_DEFAULT);
 			break;
 		case SCCP_CHANNELSTATE_CALLREMOTEMULTILINE:
 			sccp_dev_set_ringer(d, SKINNY_RINGTYPE_OFF, instance, c->callid);
@@ -360,7 +366,7 @@ void __sccp_indicate(sccp_device_t * device, sccp_channel_t * c, sccp_channelsta
 			sccp_channel_closeAllMediaTransmitAndReceive(d, c);
 			sccp_dev_starttone(d, SKINNY_TONE_REORDERTONE, instance, c->callid, 0);
 			// wait 15 seconds, then hangup automatically
-			c->scheduler.hangup = sccp_sched_add(15000, sccp_channel_sched_endcall_by_callid, &c->callid);
+			sccp_channel_schedule_hangup(c, SCCP_HANGUP_TIMEOUT);
 			break;
 		case SCCP_CHANNELSTATE_DIALING:
 			d->indicate->dialing(d, instance, c);
@@ -373,13 +379,13 @@ void __sccp_indicate(sccp_device_t * device, sccp_channel_t * c, sccp_channelsta
 			{
 				int lenDialed = 0, lenSecDialtoneDigits = 0;
 				uint32_t secondary_dialtone_tone = 0;
-				
+
 				lenDialed = strlen(c->dialedNumber);
 
 				/* secondary dialtone check */
 				lenSecDialtoneDigits = strlen(l->secondary_dialtone_digits);
 				secondary_dialtone_tone = l->secondary_dialtone_tone;
-				
+
 				if (lenSecDialtoneDigits > 0 && lenDialed == lenSecDialtoneDigits && !strncmp(c->dialedNumber, l->secondary_dialtone_digits, lenSecDialtoneDigits)) {
 					/* We have a secondary dialtone */
 					sccp_dev_starttone(d, secondary_dialtone_tone, instance, c->callid, 0);
@@ -387,7 +393,7 @@ void __sccp_indicate(sccp_device_t * device, sccp_channel_t * c, sccp_channelsta
 					sccp_dev_stoptone(d, instance, c->callid);
 				}
 			}
-		  
+
 			sccp_dev_set_keyset(d, instance, c->callid, KEYMODE_DIGITSFOLL);
 			break;
 		case SCCP_CHANNELSTATE_BLINDTRANSFER:								/* \todo SCCP_CHANNELSTATE_BLINDTRANSFER To be implemented */
@@ -407,7 +413,7 @@ void __sccp_indicate(sccp_device_t * device, sccp_channel_t * c, sccp_channelsta
 		}
 
 		/* notify features (sccp_feat_channelstateChanged = empty function, skipping) */
-		//	sccp_feat_channelstateChanged(d, c);
+		//sccp_feat_channelstateChanged(d, c);
 
 		sccp_event_t event;
 
@@ -432,9 +438,9 @@ void __sccp_indicate(sccp_device_t * device, sccp_channel_t * c, sccp_channelsta
  * \warning
  *  - line->devices is not always locked
  */
-static void __sccp_indicate_remote_device(sccp_device_t * device, sccp_channel_t * c, sccp_line_t * line, uint8_t state)
+static void __sccp_indicate_remote_device(sccp_device_t * device, sccp_channel_t * c, sccp_line_t * line, sccp_channelstate_t state)
 {
-	sccp_channel_t tmpChannel; 												/*!< use this channel to set original called/calling info */
+	sccp_channel_t tmpChannel;										/*!< use this channel to set original called/calling info */
 	int instance;
 	sccp_phonebook_t phonebookRecord = SCCP_PHONEBOOK_NONE;
 
@@ -445,12 +451,12 @@ static void __sccp_indicate_remote_device(sccp_device_t * device, sccp_channel_t
 	/** \todo move this to channel->privacy */
 	//uint32_t privacyStatus=0;
 	//if (sccp_channel_getDevice(c)) {
-	//	privacyStatus = sccp_channel_getDevic(c)->privacyFeature.status & SCCP_PRIVACYFEATURE_HINT;
+	//privacyStatus = sccp_channel_getDevic(c)->privacyFeature.status & SCCP_PRIVACYFEATURE_HINT;
 	//}
 	///* do not display private lines */
 	//if (state !=SCCP_CHANNELSTATE_CONNECTED && (c->privacy || privacyStatus > 0) ){
-	//	sccp_log((DEBUGCAT_INDICATE)) (VERBOSE_PREFIX_3 "privacyStatus status is set, ignore remote devices\n");
-	//	return;
+	//sccp_log((DEBUGCAT_INDICATE)) (VERBOSE_PREFIX_3 "privacyStatus status is set, ignore remote devices\n");
+	//return;
 	//}
 
 	/* do not propagate status of hotline */
@@ -459,7 +465,7 @@ static void __sccp_indicate_remote_device(sccp_device_t * device, sccp_channel_t
 		return;
 	}
 	sccp_linedevices_t *linedevice;
-	
+
 	memset(&tmpChannel, 0, sizeof(sccp_channel_t));
 	tmpChannel.callid = c->callid;
 	if (c->privacy || !c->callInfo.presentation) {
@@ -477,7 +483,7 @@ static void __sccp_indicate_remote_device(sccp_device_t * device, sccp_channel_t
 	tmpChannel.callInfo.presentation = c->callInfo.presentation;
 	tmpChannel.line = sccp_line_retain(c->line);
 
-	sccp_log((DEBUGCAT_INDICATE)) (VERBOSE_PREFIX_3 "%s: Indicate state %s (%d) on remote devices for channel %s (call %08x)\n", DEV_ID_LOG(device), sccp_channelstate2str(state), state, c->designator, c->callid);
+	sccp_log((DEBUGCAT_INDICATE)) (VERBOSE_PREFIX_3 "%s: Indicate state %s (%d) with reason: %s (%d) on remote devices for channel %s (call %08x)\n", DEV_ID_LOG(device), sccp_channelstate2str(state), state, sccp_channelstatereason2str(tmpChannel.channelStateReason), tmpChannel.channelStateReason, c->designator, c->callid);
 	SCCP_LIST_TRAVERSE(&line->devices, linedevice, list) {
 		if (!linedevice->device) {
 			pbx_log(LOG_NOTICE, "Strange to find a linedevice (%p) here without a valid device connected to it !", linedevice);
@@ -491,24 +497,25 @@ static void __sccp_indicate_remote_device(sccp_device_t * device, sccp_channel_t
 
 		/* check if we have one part of the remote channel */
 		AUTO_RELEASE sccp_device_t *remoteDevice = sccp_device_retain(linedevice->device);
+
 		if (remoteDevice) {
 			uint8_t stateVisibility = (c->privacy || !c->callInfo.presentation) ? SKINNY_CALLINFO_VISIBILITY_HIDDEN : SKINNY_CALLINFO_VISIBILITY_DEFAULT;
 
- 			/*! \note SKINNY_CALLINFO_VISIBILITY_HIDDEN on old devices: Dirty Hack to prevent showing the call twice (both incoming and outgoing) */
-			stateVisibility = remoteDevice->protocolversion < 17 ? SKINNY_CALLINFO_VISIBILITY_HIDDEN : stateVisibility;
+			/*! \note SKINNY_CALLINFO_VISIBILITY_HIDDEN on old devices: Dirty Hack to prevent showing the call twice (both incoming and outgoing) */
+			//    stateVisibility = remoteDevice->protocolversion < 17 ? SKINNY_CALLINFO_VISIBILITY_HIDDEN : stateVisibility;
 
-			{
-				AUTO_RELEASE sccp_channel_t *activeChannel = sccp_device_getActiveChannel(linedevice->device);
-				if (activeChannel) {
-					if (sccp_strequals(PBX(getChannelLinkedId) (activeChannel), PBX(getChannelLinkedId) (c))) {
-						sccp_log(DEBUGCAT_INDICATE)(VERBOSE_PREFIX_3 "%s: (indicate_remote_device) Already Own Part of the Call: Hidden\n", DEV_ID_LOG(device));
-						sccp_log_and(DEBUGCAT_INDICATE + DEBUGCAT_HIGH)(VERBOSE_PREFIX_3 "%s: LinkedId: %s / %s: LinkedId Remote: %s\n", DEV_ID_LOG(device), PBX(getChannelLinkedId)(c), DEV_ID_LOG(remoteDevice), PBX(getChannelLinkedId)(activeChannel));
-						continue;
-					}
+			/* Remarking the next piece out, solves the transfer issue when using sharedline as default on the transferer. Don't know why though (yet) */
+			if (state != SCCP_CHANNELSTATE_ONHOOK) {
+				AUTO_RELEASE sccp_channel_t *activeChannel = sccp_device_getActiveChannel(remoteDevice);
+
+				if (activeChannel && sccp_strequals(PBX(getChannelLinkedId) (activeChannel), PBX(getChannelLinkedId) (c))) {
+					sccp_log(DEBUGCAT_INDICATE) (VERBOSE_PREFIX_3 "%s: (indicate_remote_device) Already Own Part of the Call: Hidden\n", DEV_ID_LOG(device));
+					sccp_log_and(DEBUGCAT_INDICATE + DEBUGCAT_HIGH) (VERBOSE_PREFIX_3 "%s: LinkedId: %s / %s: LinkedId Remote: %s\n", DEV_ID_LOG(device), PBX(getChannelLinkedId) (c), DEV_ID_LOG(remoteDevice), PBX(getChannelLinkedId) (activeChannel));
+					continue;
 				}
 			}
 
-			instance = linedevice->lineInstance;//sccp_device_find_index_for_line(remoteDevice, line->name);
+			instance = linedevice->lineInstance;							//sccp_device_find_index_for_line(remoteDevice, line->name);
 			switch (state) {
 				case SCCP_CHANNELSTATE_OFFHOOK:
 					/* do nothing here, we will do the offhook simulation in CONNECTED or ONHOOK -MC */
@@ -517,34 +524,34 @@ static void __sccp_indicate_remote_device(sccp_device_t * device, sccp_channel_t
 				case SCCP_CHANNELSTATE_DOWN:
 				case SCCP_CHANNELSTATE_ONHOOK:
 					if (SKINNY_CALLTYPE_INBOUND == c->calltype && c->answered_elsewhere) {
-						switch(phonebookRecord){
+						switch (phonebookRecord) {
 							case SCCP_PHONEBOOK_RECEIVED:
 								pbx_log(LOG_NOTICE, "%s: call was answered elsewhere, record this as received call\n", DEV_ID_LOG(remoteDevice));
 								remoteDevice->indicate->remoteOffhook(remoteDevice, linedevice, c);
 								remoteDevice->indicate->connected(remoteDevice, linedevice, c);
-							break;
+								break;
 							case SCCP_PHONEBOOK_MISSED:
 								/* do nothing */
-							break;
+								break;
 							case SCCP_PHONEBOOK_NONE:
 								sccp_device_sendcallstate(remoteDevice, instance, c->callid, SKINNY_CALLSTATE_CONNECTED, SKINNY_CALLPRIORITY_LOW, SKINNY_CALLINFO_VISIBILITY_HIDDEN);
-							break;
+								break;
 						}
 					}
-					sccp_log(DEBUGCAT_INDICATE)(VERBOSE_PREFIX_3 "%s -> %s: indicate remote onhook (instance: %d, callid: %d)\n", DEV_ID_LOG(device), DEV_ID_LOG(remoteDevice), instance, c->callid);
+					sccp_log(DEBUGCAT_INDICATE) (VERBOSE_PREFIX_3 "%s -> %s: indicate remote onhook (instance: %d, callid: %d)\n", DEV_ID_LOG(device), DEV_ID_LOG(remoteDevice), instance, c->callid);
 					remoteDevice->indicate->remoteOnhook(remoteDevice, linedevice, c);
 					break;
 
 				case SCCP_CHANNELSTATE_CONNECTED:
-					switch(c->calltype){
+					switch (c->calltype) {
 #if 0
 						case SKINNY_CALLTYPE_OUTBOUND:
-							switch(phonebookRecord){
-								case SCCP_PHONEBOOK_RECEIVED:							  
+							switch (phonebookRecord) {
+								case SCCP_PHONEBOOK_RECEIVED:
 									remoteDevice->indicate->remoteOffhook(remoteDevice, linedevice, &tmpChannel);
 									remoteDevice->indicate->dialing(remoteDevice, instance, &tmpChannel);
 									remoteDevice->indicate->proceed(remoteDevice, instance, &tmpChannel);
-// 									remoteDevice->indicate->connected(remoteDevice, linedevice, &tmpChannel); /*TODO add source device to phonebook entry */
+									//    remoteDevice->indicate->connected(remoteDevice, linedevice, &tmpChannel); /*TODO add source device to phonebook entry */
 
 									sccp_copy_string(tmpChannel.callInfo.originalCalledPartyName, "originalCalledPartyName", sizeof(tmpChannel.callInfo.calledPartyNumber));
 									sccp_copy_string(tmpChannel.callInfo.originalCalledPartyNumber, "originalCalledPartyNumber", sizeof(tmpChannel.callInfo.calledPartyNumber));
@@ -553,38 +560,38 @@ static void __sccp_indicate_remote_device(sccp_device_t * device, sccp_channel_t
 									sccp_copy_string(tmpChannel.callInfo.originalCallingPartyName, "originalCalledPartyName", sizeof(tmpChannel.callInfo.calledPartyNumber));
 									sccp_copy_string(tmpChannel.callInfo.originalCallingPartyNumber, "originalCalledPartyNumber", sizeof(tmpChannel.callInfo.calledPartyNumber));
 									tmpChannel.callInfo.originalCallingParty_valid = 1;
-									
+
 									sccp_copy_string(tmpChannel.callInfo.lastRedirectingPartyName, "originalCalledPartyName", sizeof(tmpChannel.callInfo.calledPartyNumber));
 									sccp_copy_string(tmpChannel.callInfo.lastRedirectingPartyNumber, "originalCalledPartyNumber", sizeof(tmpChannel.callInfo.calledPartyNumber));
 									tmpChannel.callInfo.lastRedirectingParty_valid = 1;
-									
-									remoteDevice->indicate->connected(remoteDevice, linedevice, &tmpChannel); /*TODO add source device to phonebook entry */
-								break;
+
+									remoteDevice->indicate->connected(remoteDevice, linedevice, &tmpChannel);	/*TODO add source device to phonebook entry */
+									break;
 								case SCCP_PHONEBOOK_MISSED:
 								case SCCP_PHONEBOOK_NONE:
 									//sccp_device_sendcallstate(remoteDevice, instance, c->callid, SKINNY_CALLSTATE_CONNECTED, SKINNY_CALLPRIORITY_LOW, SKINNY_CALLINFO_VISIBILITY_HIDDEN);
-								break;
+									break;
 							}
 							break;
-#endif						
+#endif
 						case SKINNY_CALLTYPE_INBOUND:
-							switch(phonebookRecord){
-								case SCCP_PHONEBOOK_RECEIVED:							  
+							switch (phonebookRecord) {
+								case SCCP_PHONEBOOK_RECEIVED:
 									remoteDevice->indicate->remoteOffhook(remoteDevice, linedevice, &tmpChannel);
 									remoteDevice->indicate->offhook(remoteDevice, linedevice, tmpChannel.callid);
 									remoteDevice->indicate->connected(remoteDevice, linedevice, &tmpChannel);
-									
-								break;
+
+									break;
 								case SCCP_PHONEBOOK_MISSED:
-								  
-								break;
+
+									break;
 								case SCCP_PHONEBOOK_NONE:
 									sccp_device_sendcallstate(remoteDevice, instance, c->callid, SKINNY_CALLSTATE_CONNECTED, SKINNY_CALLPRIORITY_LOW, SKINNY_CALLINFO_VISIBILITY_HIDDEN);
-								break;
+									break;
 							}
 							break;
-						
-						default:						
+
+						default:
 							break;
 					}
 
@@ -593,21 +600,28 @@ static void __sccp_indicate_remote_device(sccp_device_t * device, sccp_channel_t
 					sccp_device_setLamp(remoteDevice, SKINNY_STIMULUS_LINE, instance, SKINNY_LAMP_ON);
 					sccp_device_sendcallstate(remoteDevice, instance, tmpChannel.callid, SKINNY_CALLSTATE_CALLREMOTEMULTILINE, SKINNY_CALLPRIORITY_NORMAL, stateVisibility);
 					remoteDevice->protocol->sendCallInfo(remoteDevice, &tmpChannel, instance);
-					sccp_dev_set_keyset(remoteDevice, instance, tmpChannel.callid, KEYMODE_ONHOOKSTEALABLE);
+					//  sccp_dev_set_keyset(remoteDevice, instance, tmpChannel.callid, KEYMODE_ONHOOKSTEALABLE);
+					sccp_dev_set_keyset(remoteDevice, instance, tmpChannel.callid, KEYMODE_EMPTY);	/* set NULL keymode -> No SoftKeys */
 					break;
 
 				case SCCP_CHANNELSTATE_HOLD:
-					remoteDevice->indicate->remoteHold(remoteDevice, instance, tmpChannel.callid, SKINNY_CALLPRIORITY_NORMAL, stateVisibility);
-					remoteDevice->protocol->sendCallInfo(remoteDevice, &tmpChannel, instance);
+					if (c->channelStateReason == SCCP_CHANNELSTATEREASON_NORMAL) {
+						remoteDevice->indicate->remoteHold(remoteDevice, instance, tmpChannel.callid, SKINNY_CALLPRIORITY_NORMAL, stateVisibility);
+						remoteDevice->protocol->sendCallInfo(remoteDevice, &tmpChannel, instance);
+					} else {
+						sccp_log((DEBUGCAT_INDICATE)) (VERBOSE_PREFIX_3 "%s: Skipped Remote Hold Indication for reason: %s\n", DEV_ID_LOG(device), sccp_channelstatereason2str(tmpChannel.channelStateReason));
+					}
 					break;
 
 				default:
 					break;
 
 			}
-			sccp_log((DEBUGCAT_INDICATE)) (VERBOSE_PREFIX_3 "%s: Finish Indicating state %s (%d) on remote device %s for channel %s (call %08x)\n", DEV_ID_LOG(device), sccp_channelstate2str(state), state, DEV_ID_LOG(remoteDevice), c->designator, c->callid);
+			sccp_log((DEBUGCAT_INDICATE)) (VERBOSE_PREFIX_3 "%s: Finish Indicating state %s (%d) with reason: %s (%d) on remote device %s for channel %s (call %08x)\n", DEV_ID_LOG(device), sccp_channelstate2str(state), state, sccp_channelstatereason2str(tmpChannel.channelStateReason), tmpChannel.channelStateReason, DEV_ID_LOG(remoteDevice), c->designator, c->callid);
 		}
 	}
-	
+
 	tmpChannel.line = sccp_line_release(tmpChannel.line);
 }
+
+// kate: indent-width 8; replace-tabs off; indent-mode cstyle; auto-insert-doxygen on; line-numbers on; tab-indents on; keep-extra-spaces off; auto-brackets off;
