@@ -1,23 +1,22 @@
-
 /*!
- * \file	sccp_threadpool.c
- * \brief	SCCP Threadpool Class
- * \author	Diederik de Groot < ddegroot@users.sourceforge.net >
- * \note	This program is free software and may be modified and distributed under the terms of the GNU Public License.
- * 		See the LICENSE file at the top of the source tree.
- * \note	Based on the work of Johan Hanssen Seferidis
- * 		Library providing a threading pool where you can add work. 
- * \since	2009-01-16
+ * \file        sccp_threadpool.c
+ * \brief       SCCP Threadpool Class
+ * \author      Diederik de Groot < ddegroot@users.sourceforge.net >
+ * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
+ *              See the LICENSE file at the top of the source tree.
+ * \note        Based on the work of Johan Hanssen Seferidis
+ *              Library providing a threading pool where you can add work. 
+ * \since       2009-01-16
  *
- * $Date: 2014-07-22 17:38:49 +0200 (Tue, 22 Jul 2014) $
- * $Revision: 5605 $
+ * $Date: 2014-11-05 16:40:01 +0100 (Wed, 05 Nov 2014) $
+ * $Revision: 5790 $
  */
 
 #include <config.h>
 #include "common.h"
 #include "sccp_threadpool.h"
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5605 $")
+SCCP_FILE_VERSION(__FILE__, "$Revision: 5790 $")
 #include "sccp_threadpool.h"
 #include <signal.h>
 #undef pthread_create
@@ -52,12 +51,12 @@ struct sccp_threadpool {
 /* 
  * Fast reminders:
  * 
- * tp			= threadpool 
- * sccp_threadpool	= threadpool
- * sccp_threadpool_t	= threadpool type
- * tp_p			= threadpool pointer
- * sem			= semaphore
- * xN			= x can be any string. N stands for amount
+ * tp                   = threadpool 
+ * sccp_threadpool      = threadpool
+ * sccp_threadpool_t    = threadpool type
+ * tp_p                 = threadpool pointer
+ * sem                  = semaphore
+ * xN                   = x can be any string. N stands for amount
  * */
 static volatile int sccp_threadpool_shuttingdown = 0;
 
@@ -77,7 +76,7 @@ sccp_threadpool_t *sccp_threadpool_init(int threadsN)
 		threadsN = THREADPOOL_MAX_SIZE;
 	}
 	/* Make new thread pool */
-	tp_p = (sccp_threadpool_t *) malloc(sizeof(sccp_threadpool_t));						/* MALLOC thread pool */
+	tp_p = (sccp_threadpool_t *) sccp_malloc(sizeof(sccp_threadpool_t));					/* MALLOC thread pool */
 	if (tp_p == NULL) {
 		pbx_log(LOG_ERROR, "sccp_threadpool_init(): Could not allocate memory for thread pool\n");
 		return NULL;
@@ -114,7 +113,7 @@ void sccp_threadpool_grow(sccp_threadpool_t * tp_p, int amount)
 
 	if (tp_p && !sccp_threadpool_shuttingdown) {
 		for (t = 0; t < amount; t++) {
-			tp_thread = malloc(sizeof(sccp_threadpool_thread_t));
+			tp_thread = sccp_malloc(sizeof(sccp_threadpool_thread_t));
 			if (tp_thread == NULL) {
 				pbx_log(LOG_ERROR, "sccp_threadpool_init(): Could not allocate memory for thread\n");
 				return;
@@ -144,8 +143,8 @@ void sccp_threadpool_shrink(sccp_threadpool_t * tp_p, int amount)
 		for (t = 0; t < amount; t++) {
 			tp_thread = SCCP_LIST_FIRST(&(tp_p->threads));
 			tp_thread->die = 1;
-			
-			sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "Sending die signal to thread %p in pool \n", (void *)tp_thread->thread);
+
+			sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "Sending die signal to thread %p in pool \n", (void *) tp_thread->thread);
 			// wake up all threads
 			ast_cond_broadcast(&(tp_p->work));
 		}
@@ -190,7 +189,7 @@ static void sccp_threadpool_thread_end(void *p)
 
 	ast_cond_signal(&(tp_p->exit));
 	if (res) {
-		free(res);
+		sccp_free(res);
 	}
 }
 
@@ -199,7 +198,7 @@ void sccp_threadpool_thread_do(void *p)
 {
 	sccp_threadpool_thread_t *tp_thread = (sccp_threadpool_thread_t *) p;
 	sccp_threadpool_t *tp_p = tp_thread->tp_p;
-	void * thread = (void *)pthread_self();
+	void *thread = (void *) pthread_self();
 
 	pthread_cleanup_push(sccp_threadpool_thread_end, tp_thread);
 
@@ -239,7 +238,7 @@ void sccp_threadpool_thread_do(void *p)
 			sccp_log((DEBUGCAT_THPOOL)) (VERBOSE_PREFIX_3 "(sccp_threadpool_thread_do) executing %p in thread: %p\n", job, thread);
 			if (job) {
 				func_buff(arg_buff);								/* run function */
-				free(job);									/* DEALLOC job */
+				sccp_free(job);									/* DEALLOC job */
 			}
 			// check number of threads in threadpool
 			if ((time(0) - tp_p->last_size_check) > THREADPOOL_RESIZE_INTERVAL) {
@@ -259,7 +258,7 @@ int sccp_threadpool_add_work(sccp_threadpool_t * tp_p, void *(*function_p) (void
 	if (!sccp_threadpool_shuttingdown) {
 		sccp_threadpool_job_t *newJob;
 
-		newJob = (sccp_threadpool_job_t *) malloc(sizeof(sccp_threadpool_job_t));			/* MALLOC job */
+		newJob = (sccp_threadpool_job_t *) sccp_malloc(sizeof(sccp_threadpool_job_t));			/* MALLOC job */
 		if (newJob == NULL) {
 			pbx_log(LOG_ERROR, "sccp_threadpool_add_work(): Could not allocate memory for new job\n");
 			exit(1);
@@ -335,7 +334,7 @@ void sccp_threadpool_destroy(sccp_threadpool_t * tp_p)
 	ast_cond_destroy(&(tp_p->exit));									/* Remove Condition */
 	SCCP_LIST_HEAD_DESTROY(&(tp_p->jobs));
 	SCCP_LIST_HEAD_DESTROY(&(tp_p->threads));
-	free(tp_p);												/* DEALLOC thread pool */
+	sccp_free(tp_p);											/* DEALLOC thread pool */
 	sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "Threadpool Ended\n");
 }
 
@@ -377,3 +376,5 @@ int sccp_threadpool_jobqueue_count(sccp_threadpool_t * tp_p)
 	sccp_log((DEBUGCAT_THPOOL)) (VERBOSE_PREFIX_3 "(sccp_threadpool_jobqueue_count) tp_p: %p, jobCount: %d\n", tp_p, SCCP_LIST_GETSIZE(&tp_p->jobs));
 	return SCCP_LIST_GETSIZE(&tp_p->jobs);
 }
+
+// kate: indent-width 8; replace-tabs off; indent-mode cstyle; auto-insert-doxygen on; line-numbers on; tab-indents on; keep-extra-spaces off; auto-brackets off;
